# Filename:        latex_main.R           
# Date:            2017-03-08
# Author:          Kristian Sandaker (kristian.sandaker@outlook.com)
# Purpose:         Functions for LaTeX tables and plotting
# Data Written:    data/dataset.csv
# Packages Used:   moments, stargazer, zoo
# Source Used:     helpers.R, latex_output.R

source('helpers.R')
source('latex_output.R')
source('plotters.R')
source('quantile_regression_model.R')

### Global variables ###
dataBaseName                  <- 'Database_Master_Monthly_2017-05-06_01a.csv'
groupStructureConstraintsName <- 'Groupstructure_Constraints_Master_Monthly_2017-05-06_01a.csv'
groupStructureName            <- 'Groupstructure_Master_Monthly_2017-05-06_01a.csv'
lagStructureName              <- 'Lagstructure_Master_Monthly_2017-05-06_01a.csv'
outputNamesName               <- 'Outputnames_Master_Monthly_2017-05-06_01a.csv'
timeWindowsName               <- 'Timewindows_Master_Monthly_2017-05-06_01a.csv'

minDate <- as.Date('2005-01-01')
maxDate <- as.Date('2017-01-31')

dataFrameAddress <- paste(getwd(), '/modelinput/', dataBaseName, sep='')
groupStructureDataFrameAddress <- paste(getwd(), '/modelinput/', groupStructureName, sep='')
groupStructureConstraintsDataFrameAddress <- paste(getwd(), '/modelinput/', groupStructureConstraintsName, sep='')
lagDataFrameAddress <- paste(getwd(), '/modelinput/', lagStructureName, sep='')
outputNamesDataFrameAddress <- paste(getwd(), '/modelinput/', outputNamesName, sep='')
timeWindowsDataFrameAddress <- paste(getwd(), '/modelinput/', timeWindowsName, sep='')

df_lagStructure <- importLagStructureDataFrame(lagDataFrameAddress,
                                               excludeVector=c())

# df_full <- importFullLaggedDataFrame(dataFrameAddress=dataFrameAddress,
#                                            lagDataFrameAddress=lagDataFrameAddress,
#                                            useLagDataFrame=T, maxLag=24,
#                                            startDate=minDate, endDate=maxDate,
#                                            firstColDateCol=T,
#                                            dateFormat='%d/%m/%Y',
#                                            convertToReturns='log',
#                                            forecastHorizon=12,
#                                            useTimeWindowsDataFrame=T,
#                                            timeWindowsDataFrameAddress=timeWindowsDataFrameAddress,
#                                            allowedTimeWindows=c(1,12))

# write.csv(df_full, file=paste(getwd(), '/modeloutput/', 'df_full.csv', sep=""))

df_full <- importGroupStructureDataFrame(paste(getwd(), '/modeloutput/',
                                                                 'df_full.csv', sep=''),
                                       constraints=T)
df_full <- df_full[,2:dim(df_full)[2]]

df_full <- df_full[df_full[,1] >= as.Date("2007-08-31") & df_full[,1] <= as.Date("2016-02-29"),]
df_full[,1] <- as.Date(df_full[,1])


df_groupStructure <- importGroupStructureDataFrame(groupStructureDataFrameAddress, constraints=F)
df_groupStructureConstraints <- importGroupStructureDataFrame(groupStructureConstraintsDataFrameAddress, constraints=T)
df_outputNames <- importGroupStructureDataFrame(outputNamesDataFrameAddress, constraints=F)


source('quantile_regression_model.R')
# k_populations <- qrRegGeneticAlgorithm(df_full=df_full, df_lagStructure=df_lagStructure,
#                                        df_groupStructure=df_groupStructure,
#                                        df_groupStructureConstraints=df_groupStructureConstraints,
#                                        Tau=c(0.10, 0.25, 0.50, 0.75, 0.90),
#                                        K=12, saveResult=T,
#                                        parallel=T, nCores=0)

Tau <- c(0.10, 0.25, 0.50, 0.75, 0.90)

qrModels <- constructQrModelsFromGaFile(df_full=df_full, dataFrameAddress=dataFrameAddress,
                                        gaFileName=paste(getwd(), '/modeloutput/', 'QRGA_modelsize=08_horizons=12_popsize=0230_generations=00200_variancePenalization=00050_parallel=T.csv', sep=""),
                                        Tau=Tau, maxHorizons=12, addVolatility=FALSE)
createInSampleQrSummaryTable(qrModels=qrModels, nQrTableCols=2,
                             addLandscape=FALSE,
                             metrics_decimals=2, df_outputNames=df_outputNames)
source('latex_output.R')
createInSampleQrCoefficientsTable(qrModels=qrModels, tablesPerPage=2,
                                  decimals_coefficients=3, decimals_significances=3, decimals_goodness=3,
                                  landscapeHeightMultiplier=0.90,
                                  df_outputNames=df_outputNames)

## 3D comboplot
source('plotters.R')
# plotAllCrossCorrelation3d(dataFrameAddress=dataFrameAddress)

## Exceedance plot
source("plotters.R")
latexDir <- "/Users/Kristian/Kristian/Dropbox/Apps/ShareLaTeX/Masteroppgave/plots/in-sample-exceedances"
# latexDir <- "/Users/Kristian/Kristian/Dropbox/Studier/9. semester/Prosjektoppgave/Prosjektoppgave shared/Master Thesis/Model/Data/SalmonAnalysis/plots/insample-exceedances"
plotInSampleQrExceedances(df_full, qrModels, Tau = c(0.10, 0.25, 0.50, 0.75, 0.90),
                          startDate=as.Date(df_full[2,1]), endDate=as.Date(df_full[(dim(df_full)[1]-12),1]),
                          latexDir = latexDir,
                          startDate_timeSeries = as.Date('2005-01-01'), endDate_timeSeries = as.Date('2017-01-31'),
                          outFileDir = paste(getwd(), "/plots/insample-exceedances", sep=""),
                          dataFrameAddress)

source("plotters.R")
plotInSampleQrLines(df_full=df_full, qrModels=qrModels, Tau=Tau,
                    addQuantileLines=T,
                    dataFrameAddress=dataFrameAddress)

source('latex_output.R')
createInSampleSpecificationTable(qrModels, df_outputNames=df_outputNames, qreset_orders=c(2,3,4),
                                 portraitWidthMultiplier=0.65, landscapeHeightMultiplier=0.28,
                                 addDashedSeparatorLines=F, useNpqcmstest=F,
                                 addLandscape=TRUE, nQrTableCols=2, addResizeBox=TRUE)

# Start and end dates for descriptive statistics
startDate_descStats <- as.Date("2007-08-31")
endDate_descStats   <- as.Date("2017-01-31")

source('latex_output.R')
createDescriptiveStatsTable(all=F, dataFrameAddress=dataFrameAddress,
                            outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                            lagDataFrameAddress=lagDataFrameAddress,
                            startDate_descStats=startDate_descStats, endDate_descStats=endDate_descStats,
                            outFileAddress='', useStargazer=T, removeDates=T)

source('plotters.R')
plotPriceFigures(all=F, dataFrameAddress=dataFrameAddress,
                 outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                 startDate=startDate_descStats, endDate=endDate_descStats,
                 lagDataFrameAddress=lagDataFrameAddress)

# Coefficient sensitivities
source('plotters.R')
gaFileName <- paste(getwd(), '/modeloutput/', 'QRGA_modelsize=08_horizons=12_popsize=0230_generations=00200_variancePenalization=00050_parallel=T.csv', sep="")
plotQrCoefficientSensitivities(df_full=df_full, gaFileName=gaFileName, Tau=Tau,
                               df_outputNames=df_outputNames, use_df_outputNames = TRUE,
                               maxHorizons=12)

# 3D plots with correct series naming
source('plotters.R')
plotAllCrossCorrelation3d(dataFrameAddress=dataFrameAddress,
                          outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                          startDate=as.Date('2007-08-31'), endDate=as.Date('2017-01-31'),
                          maxLag=24, maxWindow=12, forecastHorizon=12)

# df_imp <- importDataFrame(dataFrameAddress=dataFrameAddress, startDate=minDate,
#                           endDate=maxDate, firstColDateCol=T,
#                           dateFormat='%d/%m/%Y',
#                           excludeVector=excludeVector,
#                           convertToReturns='log',
#                           timeWindow=1)
# 
# write.csv(df_fullLagged, file=paste(dataDir, '/df_imp.csv', sep=""))
# 
# 
# lagDataFrameAddress <- paste(dataDir, '/', lagStructureFileName, sep='')
# 
# 
# source('plotters.R')
# plotCrossCorrelation3d(df_fullLagged, compCols=c(2,13), maxLag=24, 
#                        maxWindow=12, comboPlot=F)
# 
# 
# 
# 
# 
# 
# df_lagStructure <- importFullLaggedDataFrame(dataFrameAddress=dataFrameAddress, 
#                                             lagDataFrameAddress=lagDataFrameAddress,
#                                             useLagDataFrame=T, maxLag=24,
#                                             startDate=minDate, endDate=maxDate,
#                                             firstColDateCol=T,
#                                             dateFormat='%d/%m/%Y', excludeVector=excludeVector,
#                                             convertToReturns='log',
#                                             forecastHorizon=12,
#                                             maxWindow=12)
# write.csv(df_lagStructure, file=paste(dataDir, '/df_lagstructure.csv', sep=''))
# 
# best_lms <- bestInSampleMeanHorizonModels(df_lagStructure,
#                                           startDate_inSample=as.Date(df_fullLagged[1,1]),
#                                           endDate_inSample=as.Date(df_fullLagged[94,1]),
#                                           forecastHorizon=12, maxLmSize=10,
#                                           bss_selectionCriterion='rsq')
# 
# ctrl <- genAlgControl(populationSize = 100, numGenerations = 1500, minVariables = 5,
#                       maxVariables = 12, verbosity = 1)
# evaluatorSRCV <- evaluatorPLS(numReplications = 2, innerSegments = 7, testSetSize = 0.4,
#                               numThreads = 1)
# resultSRCV <- genAlg(df_lagStructure[,2], as.matrix(df_lagStructure[,14:dim(df_lagStructure)[2]]), control = ctrl, evaluator = evaluatorSRCV, seed = 123)
# 
# 
# df_values <- importDataFrame(dataFrameAddress, startDate=minDate,
#                              endDate=maxDate, firstColDateCol=T,
#                              dateFormat='%d/%m/%Y',
#                              convertToReturns='')
# 
# T <- dim(df_fullLagged)[1]
# 
# inSampleDates <- c(as.Date(df_fullLagged[1,1]), 
#                    as.Date("2016-02-29"))
# df_full <- df_fullLagged[df_fullLagged[,1] >= inSampleDates[1] & df_fullLagged[,1] <= inSampleDates[2],]
# 
# 
# 
# 
# res <- regsubsets(x=df_fullLagged[,3:dim(df_fullLagged)[2]], y=df_fullLagged[,2],
#            nbest=1, nvmax=10, intercept=T, method="forward")
# summary(res)
# 
# 
# ### Function calls that edits the LaTeX document ###
# df <- importDataFrame(dataFrameAddress=dataFrameAddress,
#                       startDate=minDate, endDate=maxDate, firstColDateCol=T,
#                       dateFormat='%d/%m/%Y',
#                       excludeVector=c('norwegianexports_norgessjomatrad', 
#                                       'forward.eur.Xm_fishpool'),
#                       convertToReturns='log')
# outFileAddress <- ''
# plotPriceFigures(dataFrameAddress=dataFrameAddress, startDate=minDate,
#                  endDate=maxDate)
# 
# createDescriptiveStatsTable(dataFrameAddress=dataFrameAddress,
#                             outFileAddress=outFileAddress,
#                             startDate=minDate, endDate=maxDate,
#                             removeDates=TRUE)